@@ -1,4 +1,4 @@
-/* $Id: file.c 2729 2007-01-06 14:34:57Z twogood $ */
+/* $Id$ */
 #include "internal.h"
 #if USE_OUR_OWN_MD5
 #include "md5/global.h"
@@ -77,6 +77,11 @@ static FileDescriptor* unshield_read_file_descriptor(Unshield* unshield, int ind
     case 7:
     case 8:
     case 9:
+    case 10:
+    case 11:
+    case 12:
+    case 13:
+    default:
       saved_p = p = header->data +
           header->common.cab_descriptor_offset +
           header->cab.file_table_offset +
@@ -117,10 +122,6 @@ static FileDescriptor* unshield_read_file_descriptor(Unshield* unshield, int ind
 
       assert((p - saved_p) == 0x57);
       break;
-
-    default:
-      unshield_error("Unknown major version: %i", header->major_version);
-      abort();
   }
 
   if (!(fd->flags & FILE_COMPRESSED) &&
@@ -175,7 +176,7 @@ const char* unshield_file_name (Unshield* unshield, int index)/*{{{*/
     /* XXX: multi-volume support... */
     Header* header = unshield->header_list;
 
-    return (const char*)(
+    return unshield_get_utf8_string(header, 
         header->data +
         header->common.cab_descriptor_offset +
         header->cab.file_table_offset +
@@ -363,6 +364,11 @@ static bool unshield_reader_open_volume(UnshieldReader* reader, int volume)/*{{{
     case 7:
     case 8:
     case 9:
+    case 10:
+    case 11:
+    case 12:
+    case 13:
+    default:
       {
         uint8_t six_header[VOLUME_HEADER_SIZE_V6];
         uint8_t* p = six_header;
@@ -389,10 +395,6 @@ static bool unshield_reader_open_volume(UnshieldReader* reader, int volume)/*{{{
         reader->volume_header.last_file_size_compressed_high    = READ_UINT32(p); p += 4;
       }
       break;
-
-    default:
-      abort();
-      goto exit;
   }
   
 #if VERBOSE >= 2
@@ -428,7 +430,7 @@ static bool unshield_reader_open_volume(UnshieldReader* reader, int volume)/*{{{
         /*total_bytes_left, */data_offset); 
 #endif
 
-    if (reader->index == reader->volume_header.last_file_index)
+    if (reader->index == reader->volume_header.last_file_index && reader->volume_header.last_file_offset != 0x7FFFFFFF)
     {
       /* can be first file too... */
 #if VERBOSE
@@ -452,7 +454,10 @@ static bool unshield_reader_open_volume(UnshieldReader* reader, int volume)/*{{{
       volume_bytes_left_compressed  = reader->volume_header.first_file_size_compressed;
     }
     else
-      abort();
+    {
+      success = true;
+      goto exit;
+    }
 
 #if VERBOSE
     unshield_trace("Will read 0x%08x bytes from offset 0x%08x",
