@@ -1,4 +1,4 @@
-/* $Id: internal.h 1676 2004-05-05 07:07:57Z twogood $ */
+/* $Id$ */
 #ifndef __internal_h__
 #define __internal_h__
 
@@ -16,6 +16,14 @@
 
 #include "cabfile.h"
 
+typedef struct _StringBuffer StringBuffer;
+
+struct _StringBuffer
+{
+  StringBuffer* next;
+  char* string;
+};
+
 typedef struct _Header Header;
 
 struct _Header
@@ -37,6 +45,8 @@ struct _Header
 
   int file_group_count;
   UnshieldFileGroup** file_groups;
+
+  StringBuffer* string_buffer;
 };
 
 struct _Unshield
@@ -69,6 +79,7 @@ FILE* unshield_fopen_for_reading(Unshield* unshield, int index, const char* suff
 long unshield_fsize(FILE* file);
 bool unshield_read_common_header(uint8_t** buffer, CommonHeader* common);
 
+const char* unshield_get_utf8_string(Header* header, const void* buffer);
 const char* unshield_header_get_string(Header* header, uint32_t offset);
 uint8_t* unshield_header_get_buffer(Header* header, uint32_t offset);
 
@@ -117,8 +128,18 @@ uint32_t bswap_32(uint32_t x);
 #define letoh16(x) (x)
 #endif
 
-#define READ_UINT16(p)   letoh16(*(uint16_t*)(p))
-#define READ_UINT32(p)   letoh32(*(uint32_t*)(p))
+static inline uint16_t get_unaligned_le16(const uint8_t *p)
+{
+    return p[0] | p[1] << 8;
+}
+
+static inline uint32_t get_unaligned_le32(const uint8_t *p)
+{
+    return p[0] | p[1] << 8 | p[2] << 16 | p[3] << 24;
+}
+
+#define READ_UINT16(p)   get_unaligned_le16(p)
+#define READ_UINT32(p)   get_unaligned_le32(p)
 
 #define READ_INT16(p)   ((int16_t)READ_UINT16(p))
 #define READ_INT32(p)   ((int32_t)READ_UINT32(p))
