@@ -1,4 +1,4 @@
-/* $Id: libunshield.c 1843 2004-08-24 06:10:27Z twogood $ */
+/* $Id$ */
 #define _BSD_SOURCE 1
 #include "internal.h"
 #include "log.h"
@@ -205,7 +205,7 @@ static bool unshield_header_get_file_groups(Header* header)/*{{{*/
 /**
   Read all header files
  */
-static bool unshield_read_headers(Unshield* unshield)/*{{{*/
+static bool unshield_read_headers(Unshield* unshield, int version)/*{{{*/
 {
   int i;
   bool iterate = true;
@@ -268,8 +268,22 @@ static bool unshield_read_headers(Unshield* unshield)/*{{{*/
         unshield_error("Failed to read common header from header file %i", i);
         goto error;
       }
-      
-      header->major_version = (header->common.version >> 12) & 0xf;
+
+      if (version != -1)
+      {
+        header->major_version = version;
+      }
+      else if (header->common.version >> 24 == 1)
+      {
+        header->major_version = (header->common.version >> 12) & 0xf;
+      }
+      else if (header->common.version >> 24 == 2
+               || header->common.version >> 24 == 4)
+      {
+        header->major_version = (header->common.version & 0xffff);
+        if (header->major_version != 0)
+          header->major_version = header->major_version / 100;
+      }
 
 #if 0
       if (header->major_version < 5)
@@ -325,6 +339,11 @@ error:
 }/*}}}*/
 
 Unshield* unshield_open(const char* filename)/*{{{*/
+{
+  return unshield_open_force_version(filename, -1);
+}/*}}}*/
+
+Unshield* unshield_open_force_version(const char* filename, int version)/*{{{*/
 {
   Unshield* unshield = NEW1(Unshield);
   if (!unshield)
@@ -339,7 +358,7 @@ Unshield* unshield_open(const char* filename)/*{{{*/
     goto error;
   }
 
-  if (!unshield_read_headers(unshield))
+  if (!unshield_read_headers(unshield, version))
   {
     unshield_error("Failed to read header files");
     goto error;
@@ -352,6 +371,21 @@ error:
   return NULL;
 }/*}}}*/
 
+
+static void unshield_free_string_buffers(Header* header)
+{
+  StringBuffer* current = header->string_buffer;
+  header->string_buffer = NULL;
+
+  while (current != NULL)
+  {
+    StringBuffer* next = current->next;
+    FREE(current->string);
+    FREE(current);
+    current = next;
+  }
+}
+
 void unshield_close(Unshield* unshield)/*{{{*/
 {
   if (unshield)
@@ -363,6 +397,8 @@ void unshield_close(Unshield* unshield)/*{{{*/
       Header* next = header->next;
       int i;
 
+      unshield_free_string_buffers(header);
+
 			if (header->components)
 			{
 				for (i = 0; i < header->component_count; i++)
