@@ -1,4 +1,4 @@
-/* $Id: libunshield.h,v 1.3 2004/05/07 21:01:23 twogood Exp $ */
+/* $Id$ */
 #ifndef __unshield_h__
 #define __unshield_h__
 
@@ -33,6 +33,7 @@ void unshield_set_log_level(int level);
  */
 
 Unshield* unshield_open(const char* filename);
+Unshield* unshield_open_force_version(const char* filename, int version);
 void unshield_close(Unshield* unshield);
 
 /*
@@ -83,6 +84,15 @@ bool        unshield_file_save          (Unshield* unshield, int index, const ch
 int         unshield_file_directory     (Unshield* unshield, int index);
 size_t      unshield_file_size          (Unshield* unshield, int index);
 
+/** For investigation of compressed data */
+bool unshield_file_save_raw(Unshield* unshield, int index, const char* filename);
+
+/** Maybe it's just gzip without size? */
+bool unshield_file_save_old(Unshield* unshield, int index, const char* filename);
+
+/** Deobfuscate a buffer. Seed is 0 at file start */
+void unshield_deobfuscate(unsigned char* buffer, size_t size, unsigned* seed);
+
 #ifdef __cplusplus
 }
 #endif
